@@ -5,7 +5,7 @@
 ;;   Copyright (C) 2002,2003,2004,2005 Kiyoka Nishiyama
 ;;   This program was derived from yc.el-4.0.13(auther: knak)
 ;;
-;;     $Date: 2006/06/05 14:44:01 $
+;;     $Date: 2006/08/28 15:23:36 $
 ;;
 ;; This file is part of Sumibi
 ;;
@@ -111,12 +111,12 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
   :group 'sumibi)
 
 (defcustom sumibi-use-viper nil
-  "*Non-nil $B$G$"$l$P!"(BVIPER $B$KBP1~$9$k!#(B"
+  "*Non-nil $B$G$"$l$P!"(BVIPER $B$KBP1~$9$k(B"
   :type 'boolean
   :group 'sumibi)
 
 (defcustom sumibi-realtime-guide-running-seconds 60
-  "$B%j%"%k%?%$%`%,%$%II=<($N7QB3;~4V(B($BIC?t(B)$B!&%<%m$G%,%$%II=<(5!G=$,L58z$K$J$j$^$9!#(B"
+  "$B%j%"%k%?%$%`%,%$%II=<($N7QB3;~4V(B($BIC?t(B)$B!&%<%m$G%,%$%II=<(5!G=$,L58z$K$J$k(B"
   :type  'integer
   :group 'sumibi)
 
@@ -125,6 +125,16 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
   :type  'integer
   :group 'sumibi)
 
+(defcustom sumibi-history-filename  "~/.sumibi_history"
+  "$B%f!<%6!<8GM-$NJQ49MzNr$rJ]B8$9$k%U%!%$%kL>(B"
+  :type  'string
+  :group 'sumibi)
+
+(defcustom sumibi-history-feature  t
+  "Non-nil$B$G$"$l$P!"%f!<%6!<8GM-$NJQ49MzNr$rM-8z$K$9$k(B"
+  :type  'boolean
+  :group 'sumibi)
+
 
 (defface sumibi-guide-face
   '((((class color) (background light)) (:background "#E0E0E0" :foreground "#F03030")))
@@ -154,8 +164,8 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 		  minor-mode-map-alist)))
 
 ;; $B%f!<%6!<3X=,<-=q(B
-(defvar sumibi-kakutei-history '())
-
+(defvar sumibi-kakutei-history          '())    ;; ( ( unix$B;~4V(B $BC18l(BID$B$N%j%9%H(B ) ( unix$B;~4V(B 9412 1028 ) )
+(defvar sumibi-kakutei-history-saved    '())    ;; $B%U%!%$%k$KJ]B8$5$l$?$[$&$N%R%9%H%j%G!<%?(B)
 
 ;;;
 ;;; hooks
@@ -164,8 +174,14 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 (defvar sumibi-select-mode-hook nil)
 (defvar sumibi-select-mode-end-hook nil)
 
+(defconst sumibi-kind-index   0)
+(defconst sumibi-tango-index  1)
+(defconst sumibi-id-index     2)
+(defconst sumibi-wordno-index 3)
+(defconst sumibi-candno-index 4)
+(defconst sumibi-spaces-index 5)
 
-(defvar sumibi-hiragana->katakana-table
+(defconst sumibi-hiragana->katakana-table
   (mapcar
    (lambda (c)
      (cons
@@ -197,7 +213,7 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
      "$B$C$s(B"))))
 
 
-(defvar sumibi-roman->kana-table
+(defconst sumibi-roman->kana-table
   '(("kkya" . "$B$C$-$c(B")
     ("kkyu" . "$B$C$-$e(B")
     ("kkyo" . "$B$C$-$g(B")
@@ -511,6 +527,7 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 (defvar sumibi-timer    nil)            ; $B%$%s%?!<%P%k%?%$%^!<7?JQ?t(B
 (defvar sumibi-timer-rest  0)           ; $B$"$H2?2s8F=P$5$l$?$i!"%$%s%?!<%P%k%?%$%^$N8F=P$r;_$a$k$+(B
 (defvar sumibi-guide-overlay   nil)     ; $B%j%"%k%?%$%`%,%$%I$K;HMQ$9$k%*!<%P!<%l%$(B
+(defvar sumibi-last-request-time 0)     ; Sumibi$B%5!<%P!<$K%j%/%(%9%H$7$?:G8e$N;~9o(B($BC10L$OIC(B)
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -528,6 +545,21 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 ;;
 (defun sumibi-init ()
 
+  ;; $B%R%9%H%j%U%!%$%k$H%a%b%jCf$N%R%9%H%j%G!<%?$r%^!<%8$9$k(B
+  (defun sumibi-merge-kakutei-history (base-list new-list)
+    (let ((merged-num  '())
+	  (merged-list '()))
+      (mapcar
+       (lambda (x)
+	 (when (not (member (car x) merged-num))
+	   (progn
+	     (push (car x) merged-num)
+	     (push      x  merged-list))))
+       (append
+	base-list
+	new-list))
+      merged-list))
+
   ;; $B%F%s%]%i%j%U%!%$%k$r:n@.$9$k!#(B
   (defun sumibi-make-temp-file (base)
     (if	(functionp 'make-temp-file)
@@ -544,9 +576,30 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
       (insert sumibi-server-cert-data)
       (write-region (point-min) (point-max) sumibi-server-cert-file  nil nil))
 
+    (when (and
+	   sumibi-history-feature
+	   (file-exists-p sumibi-history-filename))
+      (progn
+	(load-file sumibi-history-filename)
+	(setq sumibi-kakutei-history sumibi-kakutei-history-saved)))
+
     ;; Emacs$B=*N;;~(BSSL$B>ZL@=q%U%!%$%k$r:o=|$9$k!#(B
     (add-hook 'kill-emacs-hook
 	      (lambda ()
+		;; $B%f!<%6!<JQ49MzNr$r%^!<%8$7$FJ]B8$9$k(B
+		(when sumibi-history-feature
+		  (progn
+		    ;; $B8=:_$N%U%!%$%k$r:FEYFI$_$3$`(B($BJL$N(BEmacs$B%W%m%;%9$,99?7$7$F$$$k$+$bCN$l$J$$0Y(B)
+		    (when (file-exists-p sumibi-history-filename)
+		      (load-file sumibi-history-filename))
+		    (with-temp-file
+			sumibi-history-filename
+		      (insert (format "(setq sumibi-kakutei-history-saved '%s)" 
+				      (pp-to-string 
+				       (sumibi-merge-kakutei-history
+					sumibi-kakutei-history-saved
+					sumibi-kakutei-history)))))))
+		;; SSL$B>ZL@=q$N%U%!%$%k$r:o=|$9$k(B
 		(delete-file sumibi-server-cert-file)))
 
     ;; $B=i4|2=40N;(B
@@ -613,6 +666,18 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 	_xml))))
 
       
+;;
+;; $B8=:_;~9o$r(BUNIX$B%?%$%`$rJV$9(B($BC10L$OIC(B)
+;;
+(defun sumibi-current-unixtime ()
+  (let (
+	(_ (current-time)))
+    (+
+     (* (car _)
+	65536)
+     (cadr _))))
+
+
 ;;
 ;; $B%m!<%^;z$G=q$+$l$?J8>O$r(BSumibi$B%5!<%P!<$r;H$C$FJQ49$9$k(B
 ;;
@@ -621,18 +686,21 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 
   (message "Requesting to sumibi server...")
   (let* (
-	 (result (sumibi-soap-request "doSumibiConvertSexp" (list yomi))))
-
+	 (result (sumibi-soap-request "doSumibiConvertSexp" (list yomi
+								  ""
+								  (sumibi-get-history-string
+								   sumibi-kakutei-history)))))
     (sumibi-debug-print (format "henkan-result:%S\n" result))
     (if (eq (string-to-char result) ?\( )
 	(progn
-	 (message nil)
-	 (condition-case err
-	     (read result)
-	   (end-of-file
-	    (progn
-	      (message "Parse error for parsing result of Sumibi Server.")
-	      nil))))
+	  (sumibi-next-history)
+	  (message nil)
+	  (condition-case err
+	      (read result)
+	    (end-of-file
+	     (progn
+	       (message "Parse error for parsing result of Sumibi Server.")
+	       nil))))
       (progn
 	(message result)
 	nil))))
@@ -679,41 +747,29 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 	   (henkan-list (sumibi-henkan-request (sumibi-replace-keyword yomi))))
       
       (if henkan-list
-	  (progn
-	    ;; $BJ8@a$,0l$D$N>l9g$@$1%f!<%6!<<-=q$rMxMQ$9$k(B
-	    (when (>= 1 (length henkan-list))
-	      (let ( 
-		    (_ (assoc yomi sumibi-kakutei-history)))
-		(when _
-		  (setq henkan-list
-			(list
-			 (append
-			  `((j ,(cdr _) 0 0 0))
-			  (car henkan-list)))))))
+	  (condition-case err
+	      (progn
+		(setq
+		 ;; $BJQ497k2L$NJ];}(B
+		 sumibi-henkan-list henkan-list
+		 ;; $BJ8@aA*Br=i4|2=(B
+		 sumibi-cand-n   (make-list (length henkan-list) 0)
+		 ;; 
+		 sumibi-cand-max (mapcar
+				  (lambda (x)
+				    (length x))
+				  henkan-list))
 		
-	    (condition-case err
-		(progn
-		  (setq
-		   ;; $BJQ497k2L$NJ];}(B
-		   sumibi-henkan-list henkan-list
-		   ;; $BJ8@aA*Br=i4|2=(B
-		   sumibi-cand-n   (make-list (length henkan-list) 0)
-		   ;; 
-		   sumibi-cand-max (mapcar
-				    (lambda (x)
-				      (length x))
-				    henkan-list))
-		  
-		  (sumibi-debug-print (format "sumibi-henkan-list:%s \n" sumibi-henkan-list))
-		  (sumibi-debug-print (format "sumibi-cand-n:%s \n" sumibi-cand-n))
-		  (sumibi-debug-print (format "sumibi-cand-max:%s \n" sumibi-cand-max))
-		  ;;
-		  t)
-	      (sumibi-trap-server-down
-	       (beep)
-	       (message (error-message-string err))
-	       (setq sumibi-select-mode nil))
-	      (run-hooks 'sumibi-select-mode-end-hook)))
+		(sumibi-debug-print (format "sumibi-henkan-list:%s \n" sumibi-henkan-list))
+		(sumibi-debug-print (format "sumibi-cand-n:%s \n" sumibi-cand-n))
+		(sumibi-debug-print (format "sumibi-cand-max:%s \n" sumibi-cand-max))
+		;;
+		t)
+	    (sumibi-trap-server-down
+	     (beep)
+	     (message (error-message-string err))
+	     (setq sumibi-select-mode nil))
+	    (run-hooks 'sumibi-select-mode-end-hook))
 	nil))))
 
 
@@ -768,7 +824,7 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
        (let ((word (nth (nth cnt sumibi-cand-n) x)))
 	 (sumibi-debug-print (format "word:[%d] %s\n" cnt word))
 	 (setq cnt (+ 1 cnt))
-	 (cadr word)))
+	 (nth sumibi-tango-index word)))
      sumibi-henkan-list
      "")))
 
@@ -808,8 +864,8 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 		(start       (point-marker))
 		(_n          (nth cnt sumibi-cand-n))
 		(_max        (nth cnt sumibi-cand-max))
-		(spaces      (nth 4 (nth _n x)))
-		(insert-word (cadr (nth _n x)))
+		(spaces      (nth sumibi-spaces-index (nth _n x)))
+		(insert-word (nth sumibi-tango-index  (nth _n x)))
 		(_insert-word
 		 ;; $B%9%Z!<%9$,(B2$B8D0J>eF~$l$i$l$?$i!"(B1$B8D$N%9%Z!<%9$rF~$l$k!#(B($BC"$7!"(Bauto-fill-mode$B$,L58z$N>l9g$N$_(B)
 		 (if (and (< 1 spaces) (not auto-fill-function))
@@ -820,12 +876,7 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 		 (if (eq cnt sumibi-cand)
 		     (progn
 		       (insert _insert-word)
-		       (message (format "[%s] candidate (%d/%d)" insert-word (+ _n 1) _max))
-		       ;; $B%f!<%6!<<-=q$KEPO?$9$k(B
-		       (setq sumibi-kakutei-history 
-			     (append
-			      `((,ank-word . ,insert-word))
-			      sumibi-kakutei-history)))
+		       (message (format "[%s] candidate (%d/%d)" insert-word (+ _n 1) _max)))
 		   (insert _insert-word)))
 		(end         (point-marker))
 		(ov          (make-overlay start end)))
@@ -889,6 +940,10 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 (define-key sumibi-select-mode-map "j"                      'sumibi-select-kanji)
 (define-key sumibi-select-mode-map "h"                      'sumibi-select-hiragana)
 (define-key sumibi-select-mode-map "k"                      'sumibi-select-katakana)
+(define-key sumibi-select-mode-map "u"                      'sumibi-select-hiragana)
+(define-key sumibi-select-mode-map "i"                      'sumibi-select-katakana)
+(define-key sumibi-select-mode-map "\C-u"                   'sumibi-select-hiragana)
+(define-key sumibi-select-mode-map "\C-i"                   'sumibi-select-katakana)
 (define-key sumibi-select-mode-map "l"                      'sumibi-select-alphabet)
 
 
@@ -906,6 +961,63 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
    (marker-position sumibi-fence-end)
    sumibi-select-mode))
 
+
+;; $B3NDj$7$?(BID$B%j%9%H$rJQ49MzNr$KDI2C$9$k(B
+(defun sumibi-next-history ( )
+  (if sumibi-history-feature
+      (progn
+	(push
+	 (cons 
+	  (sumibi-current-unixtime)
+	  '())
+	 sumibi-kakutei-history)
+	(sumibi-debug-print (format "init:kakutei-history:%S\n" sumibi-kakutei-history))
+	sumibi-kakutei-history)
+    '()))
+
+
+;; Sumibi$B%5!<%P!<$K(B $BAw$k%R%9%H%j%j%9%H$r=P$9(B
+(defun sumibi-get-history-string (kakutei-history)
+  (mapconcat
+   (lambda (entry)
+     (mapconcat
+      (lambda (x) (number-to-string x))
+      (cdr entry)
+      " "))
+   kakutei-history
+   ";"))
+
+;;(sumibi-get-history-string
+;; '(
+;;   (1 2 3 4 5 6)
+;;   (10 20 30 40 50 60)))
+
+;; $B3NDj$7$?(BID$B%j%9%H$r99?7$9$k(B
+(defun sumibi-update-history( cand-n )
+  (let* ((cnt 0)
+	 (result 
+	  (mapcar
+	   (lambda (x)
+	     ;; $BJQ497k2LJ8;zNs$rJV$9!#(B
+	     (let ((word (nth (nth cnt cand-n) x)))
+	       (sumibi-debug-print (format "history-word:[%d] %s\n" cnt word))
+	       (setq cnt (+ 1 cnt))
+	       (nth sumibi-id-index word)))
+	   sumibi-henkan-list)))
+    ;; $B%R%9%H%j%G!<%?$r:n$jD>$9(B
+    (when sumibi-history-feature
+      (setq sumibi-kakutei-history
+	    (cons
+	     (cons
+	      (caar sumibi-kakutei-history)
+	      result)
+	     (if (< 1 (length sumibi-kakutei-history))
+		 (cdr sumibi-kakutei-history)
+	       '())))
+      (sumibi-debug-print (format "kakutei-history:%S\n" sumibi-kakutei-history)))))
+  
+
+
 ;; $B8uJdA*Br$r3NDj$9$k(B
 (defun sumibi-select-kakutei ()
   "$B8uJdA*Br$r3NDj$9$k(B"
@@ -914,7 +1026,9 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
   (setq sumibi-cand-n-backup (copy-list sumibi-cand-n))
   (setq sumibi-select-mode nil)
   (run-hooks 'sumibi-select-mode-end-hook)
-  (sumibi-select-update-display))
+  (sumibi-select-update-display)
+  (sumibi-update-history sumibi-cand-n))
+
 
 ;; $B8uJdA*Br$r%-%c%s%;%k$9$k(B
 (defun sumibi-select-cancel ()
@@ -993,7 +1107,7 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 
     ;; $BO"A[%j%9%H$+$i(B _type $B$G0z$$$?(B index $BHV9f$r@_Dj$9$k$@$1$GNI$$!#(B
     (when _element
-      (setcar (nthcdr n sumibi-cand-n) (nth 3 _element))
+      (setcar (nthcdr n sumibi-cand-n) (nth sumibi-candno-index _element))
       (sumibi-select-update-display))))
 
 (defun sumibi-select-kanji ()
@@ -1168,8 +1282,8 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 		     start-point
 		     (skip-chars-forward (concat "\t " sumibi-stop-chars) (point-at-eol))))))
 
-	  (sumibi-debug-print (format "(point) = %d  result = %d  limit-point = %d\n" (point) result limit-point))
-	  (sumibi-debug-print (format "a = %d b = %d \n" (+ (point) result) limit-point))
+	  ;; (sumibi-debug-print (format "(point) = %d  result = %d  limit-point = %d\n" (point) result limit-point))
+	  ;; (sumibi-debug-print (format "a = %d b = %d \n" (+ (point) result) limit-point))
 
 	  ;; $B%Q%i%0%i%U0LCV$G%9%H%C%W$9$k(B
 	  (if (< (+ (point) result) limit-point)
@@ -1189,8 +1303,8 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 		   start-point
 		   (skip-chars-forward (concat "\t " sumibi-stop-chars) (point-at-eol))))))
 
-	(sumibi-debug-print (format "(point) = %d  result = %d  limit-point = %d\n" (point) result limit-point))
-	(sumibi-debug-print (format "a = %d b = %d \n" (+ (point) result) limit-point))
+	;; (sumibi-debug-print (format "(point) = %d  result = %d  limit-point = %d\n" (point) result limit-point))
+	;; (sumibi-debug-print (format "a = %d b = %d \n" (+ (point) result) limit-point))
 
 	(if (< (+ (point) result) limit-point)
 	    ;; $B%$%s%G%s%H0LCV$G%9%H%C%W$9$k!#(B
@@ -1445,7 +1559,7 @@ point $B$+$i9TF,J}8~$KF1<o$NJ8;zNs$,B3$/4V$r4A;zJQ49$7$^$9!#(B
 (setq default-input-method "japanese-sumibi")
 
 (defconst sumibi-version
-"0.7.0"
+"0.7.1"
   )
 (defun sumibi-version (&optional arg)
   "$BF~NO%b!<%IJQ99(B"
