@@ -10,6 +10,8 @@
    sumibi-evaluate-balance
    sumibi-candidate-top-limit
    sumibi-evaluate-okurigana
+   sumibi-dict-max-characters
+   sumibi-word-times-rate
    sumibi-interface
    sumibi-method-list
    sumibi-sumibidb-host
@@ -20,14 +22,13 @@
    sumibi-sumiyakidb-name
    sumibi-sumiyakidb-user
    sumibi-sumiyakidb-password))
-
 (select-module sumibi.define)
 
 
 (define sumibi-debug #f)
 
 (define sumibi-version 
-"0.7.0"
+"0.7.1"
   )
 
 ;; 評価バランス ( skip-2重マルコフ 2重マルコフ 1重マルコフ 2重マルコフ skip-2重マルコフ)
@@ -42,6 +43,11 @@
 ;; 外部との動作インターフェースモード ( 'std  か 'cgi )
 (define sumibi-interface 'std)
 
+;; 辞書に登録する単語の最大文字数
+(define sumibi-dict-max-characters 30)
+
+;; ユーザー履歴中の頻度に対する重み(何倍するか)
+(define sumibi-word-times-rate 2)
 
 ;;
 ;; DBサーバーに接続するためのパラメータ ( DBサーバーソフトウェアはMySQL固定 )
